@@ -38,7 +38,8 @@ public:
 		ErrorConnect,
 		ErrorTls,
 		ErrorTimeout,
-		ErrorBodyNotAllowed
+		ErrorBodyNotAllowed,
+		ErrorTooManyRedirects
 	};
 
 	HttpRequest(QJDnsShared *dns, QObject *parent = 0);
@@ -46,8 +47,9 @@ public:
 
 	void setConnectHost(const QString &host);
 	void setIgnoreTlsErrors(bool on);
+	void setFollowRedirects(int maxRedirects); // -1 to disable
 
-	void start(const QString &method, const QUrl &uri, const HttpHeaders &headers = HttpHeaders());
+	void start(const QString &method, const QUrl &uri, const HttpHeaders &headers = HttpHeaders(), bool willWriteBody = true);
 
 	// may call this multiple times
 	void writeBody(const QByteArray &body);
